@@ -462,12 +462,8 @@ function (struct command *current)	/* performs a function */
     case fSTR2:
     case fSTR3:
         result = stSTRING;
-        if (!myformat
-                (string, a1->value, a2->pointer, a3 ? a3->pointer : NULL)) {
-            pointer = my_strdup ("");
-            sprintf (string, "'%s' is not a valid format",
-                     (char *) a2->pointer);
-            error (sERROR, string);
+        if (!myformat (string, INBUFFLEN, a1->value, a2->pointer, a3 ? a3->pointer : NULL)) {
+            pointer = my_strdup (""); 
             break;
         }
         pointer = my_strdup (string);
@@ -502,6 +498,10 @@ function (struct command *current)	/* performs a function */
 	value = floor(a1->value);
         result = stNUMBER;
         break;
+    case fROUND:
+	value = round(a1->value);
+        result = stNUMBER;
+        break;
     case fFRAC:
         if (a1->value < 0) {
             value = a1->value + floor (-a1->value);
@@ -595,15 +595,27 @@ function (struct command *current)	/* performs a function */
         result = stSTRING;
         break;
     case fAND:
-        value = (int) a1->value & (int) a2->value;
+        value = (unsigned int) a1->value & (unsigned int) a2->value;
         result = stNUMBER;
         break;
     case fOR:
-        value = (int) a1->value | (int) a2->value;
+        value = (unsigned int) a1->value | (unsigned int) a2->value;
         result = stNUMBER;
         break;
     case fEOR:
-        value = (int) a1->value ^ (int) a2->value;
+        value = (unsigned int) a1->value ^ (unsigned int) a2->value;
+        result = stNUMBER;
+        break;
+    case fBITNOT:
+        value = ~ (unsigned int) a1->value;
+        result = stNUMBER;
+        break;
+    case fSHL:
+        value = (unsigned int) a1->value << (int) a2->value;
+        result = stNUMBER;
+        break;
+    case fSHR:
+        value = (unsigned int) a1->value >> (int) a2->value;
         result = stNUMBER;
         break;
     case fMOUSEX:
@@ -1209,13 +1221,24 @@ other2dec (char *hex, int base)	/* convert hex or binary to double number */
 
 
 int
-myformat (char *dest, double num, char *format, char *sep)	/* format number according to string */
+myformat (char *dest, int max, double num, char *format, char *sep)	/* format number according to string */
 {
-    int i1, i2;			/* dummy */
-    char c1;			/* dummy */
-    static char ctrl[6];
+    int ret = myformat2 (dest, max, num, format, sep);
+    if (ret == 0) return TRUE;
+    if (ret == 1) sprintf (estring, "'%s' is not a valid format", format);
+    if (ret == 2) sprintf (estring, "length of formatted string exceeds maximum of %d bytes", INBUFFLEN);
+    error (sERROR, estring);
+    return FALSE;
+}
+
+
+int
+myformat2 (char *dest, int max, double num, char *format, char *sep)	/* do the work for myformat */
+{
+    static char *ctrl = "+- #0";	/* allowed control chars for c-format */
+    char formchar;
     char *found, *form;
-    int pre, post, len, i, digit, colons, dots;
+    int pre, post, len, nread, digit, colons, dots, i;
     int neg = FALSE;
     double ip, fp, round;
     static char *digits = "0123456789";
@@ -1223,23 +1246,23 @@ myformat (char *dest, double num, char *format, char *sep)	/* format number acco
     form = format;
     if (*form == '%') {
         /* c-style format */
-        strcpy (ctrl, "+- #0");	/* allowed control chars for c-format */
         form++;
-        while ((found = strchr (ctrl, *form)) != NULL) {
-            *found = '?';
-            form++;
+        while ((found = strchr (ctrl, *form)) != NULL) form++;
+        if (sscanf (form, "%*u.%*u%c%n", &formchar, &nread) != 1 &&
+	    sscanf (form, "%*u.%c%n", &formchar, &nread) != 1 &&
+	    sscanf (form, ".%*u%c%n", &formchar, &nread) != 1 &&
+	    sscanf (form, "%*u%c%n", &formchar, &nread) != 1 &&
+	    sscanf (form, "%c%n", &formchar, &nread) != 1) {
+            return 1;
         }
-        if (sscanf (form, "%u.%u%c%n", &i1, &i2, &c1, &i) != 3 &&
-                sscanf (form, "%u.%c%n", &i2, &c1, &i) != 2 &&
-                sscanf (form, ".%u%c%n", &i2, &c1, &i) != 2 &&
-                sscanf (form, "%u%c%n", &i2, &c1, &i) != 2) {
-            return FALSE;
+        if (!strchr ("feEgG", formchar) || form[nread]) {
+            return 1;
         }
-        if (!strchr ("feEgG", c1) || form[i]) {
-            return FALSE;
-        }
-        /* seems okay, let's print */
-        sprintf (dest, format, num);
+        /* seems okay, let's try to print */
+        len = snprintf (dest, max, format, num);
+	if (len >= max) {
+	    return 2;
+	}
     } else {
         /* basic-style format */
         if (num < 0) {
@@ -1253,7 +1276,7 @@ myformat (char *dest, double num, char *format, char *sep)	/* format number acco
         for (form = format; *form; form++) {
             if (*form == ',') {
                 if (dots) {
-                    return FALSE;
+                    return 1;
                 }
                 colons++;
             } else if (*form == '.') {
@@ -1265,11 +1288,11 @@ myformat (char *dest, double num, char *format, char *sep)	/* format number acco
                     pre++;
                 }
             } else {
-                return FALSE;
+                return 1;
             }
         }
         if (dots > 1) {
-            return FALSE;
+            return 1;
         }
         len = strlen (format);
         dest[len] = '\0';
@@ -1316,7 +1339,7 @@ myformat (char *dest, double num, char *format, char *sep)	/* format number acco
         }
         if ((neg && i < 0) || ((int) ip)) {
             strcpy (dest, format);
-            return TRUE;
+            return 0;
         }
         if (neg) {
             dest[i--] = '-';
@@ -1341,7 +1364,7 @@ myformat (char *dest, double num, char *format, char *sep)	/* format number acco
             }
         }
     }
-    return TRUE;
+    return 0;
 }
 
 
