@@ -117,6 +117,7 @@ main (int argc, char **argv)
     DWORD consoleflags;
 #endif
     int len;
+    char lpath_def_from;
 
     string = (char *) my_malloc (sizeof (char) * INBUFFLEN);
     estring = (char *) my_malloc (sizeof (char) * INBUFFLEN);
@@ -171,22 +172,29 @@ main (int argc, char **argv)
     }
 
 #endif
-    /* get library path */
+
+    /* compute default for library path before parsing arguments, because we want to report it within the usage message */
+    /* remark: no debuglevel known yet, so we defer debug-messages until after parsing of command line args */
     library_path[0] = '\0';
     library_default[0] = '\0';
 #ifdef UNIX
     strcpy (library_default, LIBRARY_PATH);
+    lpath_def_from = 'b';
 #else
     fromlibpath = TRUE;
     if (lp = getreg ("librarypath")) {
         strcpy (library_default, lp);
         fromlibpath = TRUE;
+	lpath_def_from = 'l';
     } else if (lp = getreg ("path")) {
         strcpy (library_default, lp);
         fromlibpath = FALSE;
+	lpath_def_from = 'p';
     } else {
         library_default[0] = '\0';
         fromlibpath = FALSE;
+	lpath_def_from = 'n';
+	if (severity_threshold <= sDEBUG) error (sDEBUG, "No default for library_path ");
     }
 #endif
 
@@ -197,10 +205,34 @@ main (int argc, char **argv)
     /* parse arguments */
     parse_arguments (argc, argv);
 
+    /* now, that we have the debuglevel, we may report, where we got the library path from */
+    if (severity_threshold <= sDEBUG) {
+      switch(lpath_def_from) {
+      case 'b':
+	sprintf (string, "Using builtin default for library_path: %s", library_default);
+	break;
+      case 'l':
+	sprintf (string, "Using default for library_path from registry key 'librarypath': %s", library_default);
+	break;
+      case 'p':
+	sprintf (string, "Using default for library_path from registry key 'path': %s", library_default);
+	break;
+      case 'n':
+	sprintf (string, "No default for library_path ");
+	break;
+      }
+      error (sDEBUG, string);
+    }
+    
     /* brush up library path */
     if (!library_path[0]) {
         strcpy (library_path, library_default);
+	if (severity_threshold <= sDEBUG) error (sDEBUG, "No library_path specified on commandline");
+    } else if  (severity_threshold <= sDEBUG) {
+      sprintf (string, "library_path from commandline is: %s", library_path);
+      error (sDEBUG, string);
     }
+
     len = strlen (library_path);
 #ifdef UNIX
     if (library_path[len - 1] == '/' || library_path[len - 1] == '\\') {
@@ -216,6 +248,10 @@ main (int argc, char **argv)
         }
     }
 #endif
+    if (severity_threshold <= sDEBUG) {
+      sprintf (string, "Final value for library_path is: %s", library_path);
+      error (sDEBUG, string);
+    }
 
     compilation_start = compilation_end  = current_millis();
 
@@ -242,7 +278,7 @@ main (int argc, char **argv)
 	    printf
 		("Your program will execute immediately and cannot be saved;\n");
 	    printf
-		("create your program with an external editor, if you want to keep it.\n");
+		("create your program with an external editor, to avoid this.\n");
 #ifdef UNIX
 	    printf
 		("Type 'man yabasic' or see the file yabasic.htm for more information,\n");
@@ -335,7 +371,7 @@ std_diag (char *head, int type, char *symname, char *diag)	/* produce standard d
 	}
     }
     dest += n;
-    if (stackhead->prev != stackroot) {
+    if (stackhead->prev != stackroot && stackhead != stackroot) {
 	sprintf (dest, "t[");
 	dest += 2;
 	sp = stackhead;
@@ -1323,6 +1359,7 @@ initialize (void)
     fexplanation[fSQRT] = "SQRT";
     fexplanation[fSQR] = "SQR";
     fexplanation[fFRAC] = "FRAC";
+    fexplanation[fROUND] = "ROUND";
     fexplanation[fABS] = "ABS";
     fexplanation[fSIG] = "SIG";
     fexplanation[fRAN] = "RAN";
@@ -1341,6 +1378,7 @@ initialize (void)
     fexplanation[fRTRIM] = "RTRIM";
     fexplanation[fTRIM] = "TRIM";
     fexplanation[fCHR] = "CHR";
+    fexplanation[fBITNOT] = "BITNOT";
     fexplanation[fONEARGS] = "ONEARGS";
     fexplanation[fDEC2] = "DEC2";
     fexplanation[fATAN2] = "ATAN2";
@@ -1348,6 +1386,8 @@ initialize (void)
     fexplanation[fAND] = "AND";
     fexplanation[fOR] = "OR";
     fexplanation[fEOR] = "EOR";
+    fexplanation[fSHL] = "SHL";
+    fexplanation[fSHR] = "SHR";
     fexplanation[fLOG2] = "LOG2";
     fexplanation[fRIGHT] = "RIGHT";
     fexplanation[fINSTR] = "INSTR";
@@ -1564,7 +1604,7 @@ run_it ()
             case cEND_SWITCH_MARK:
                 DONE;
             case cERROR:
-		error (sERROR, pop (stSTRING)->pointer);
+		error_without_position (sERROR, pop (stSTRING)->pointer);
                 DONE;
             case cCOMPILE:
                 compile ();
@@ -1953,7 +1993,7 @@ error_with_position (int severity, char *message, char *filename, int lineno, in
 	if (filename && severity == sERROR) {
 	    show_and_mark_line (filename, lineno, first_column, last_column);
 	}
-        if (severity >= sERROR && !sub_dumped) {
+        if (severity >= sERROR && !sub_dumped && program_state > spINITIALIZED) {
             dump_sub (1);
 	    sub_dumped = TRUE;
         }
@@ -2411,7 +2451,7 @@ seekback (FILE *file, int offset, int cookie_found)           /* seek back bytes
     error (cookie_found ? sWARNING:sDEBUG, "Could not read from end of embedded program");
     return FALSE;
   }
-  string[strlen(string) - strlen("\n")] = '\0';
+  string[(strlen(string) || 1) - 1] = '\0';
   if (cookie_found && severity_threshold <= sDEBUG) { 
     sprintf(estring, "Next line from end of embbeded program to be processed is: '%s'", string);
     error (sDEBUG, estring);
@@ -2516,7 +2556,7 @@ mybind (char *bound)		/* bind a program to the interpreter and save it */
     proglen += 5;
     fprintf (fbound, "rem %08d\n", proglen);
     fprintf (fbound, "rem %s\n", progname);
-    fprintf (fbound, "rem %08d\n", strlen(progname));
+    fprintf (fbound, "rem %08lu\n", strlen(progname));
     fprintf (fbound, "rem %02d\n", severity_threshold + yydebug);
     fprintf (fbound, "rem %s\n", YABMAGIC);
     fclose (fyab);
